<?php
// +----------------------------------------------------------------------
// | 2、登出
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace app\socket\controller\action;
use think\Db;

class logout {
    // 连接池实例
    private $redis_obj;
    private $mysql_obj;

    // 构造函数创建连接池
    public function __construct() {
        $this->redis_obj = \pool\RedisPool::run()->pop();
        $this->mysql_obj = \pool\MysqlPool::run()->write_pop();
    }
    
    // 利用析构函数归回连接，并消耗内存
    public function __destruct() {
        # 归回连接
        \pool\MysqlPool::run()->write_free($this->mysql_obj);
        # 归还连接
        \pool\RedisPool::run()->free($this->redis_obj);
        # 销毁内存
        unset($this->mysql_obj);
        unset($this->redis_obj);
    }
   
    /**
     * 统一入口
     * @todo 无
     * @author 小黄牛
     * @version v1.0.9 + 2020.04.16
     * @deprecated 暂不启用
     * @global 无
     * @param swoole $ws 实例
     * @param swoole $request 请求体信息
     * @param array $param 数据包
     * @return void
    */
    public function run($ws, $request, $param=[]) {
        $redis = $this->redis_obj['redis'];
        $db = $this->mysql_obj['db'];
        # 获取动态数据
        $info =  $redis->hgetall($ws->config['websocket_redis_table'].$request->fd);
   
        if (!$info) {
            \callback\error::run($ws, $request->fd, '非法请求');
            # 归还连接
            \pool\RedisPool::run()->free($redis_obj);
            return false;
        }

        # 启动事务
        $res = $db->begin();
        if ($res) {
            # 修改数据下班
            try {
                $sql = \db\SwooleDb::run()->table('user')->where('id', $info['id'])->update([
                    'client_fd' => '',
                    'is_work_status' => 0
                ]);
                # 执行写入操作
                $ret = $db->query($sql);
                # 提交事务
                $db->commit();
            } catch (\Exception $e) {
                #回滚事务
                $db->rollback();
                \callback\error::run($ws, $request->fd, '服务器异常');
                return false;
            }
        }
        # 更新swoole缓存信息
        # 销毁内存存储器
        $ws->user->del($info['id']);
        # 销毁redis
        $redis->hdel($ws->config['websocket_redis_table'].$request->fd);
        # 推送下线消息
        \callback\success::run($ws, $request->fd, '退出成功');
        # 销毁连接
        $ws->close($request->fd);
    }
    
    /**
     * 该接口文档说明
     * @author 小黄牛
     * @version v1.0.1 + 2020.02.24
    */
    public function _link() {
        return [
            // 请求参数
            'param' => [
            ],
            // 返回值
            'return' => [
                // 状态码囊括
                'action' => [
                    'success' => '请求成功', 
                    'error' => '接口错误',
                ]
            ]
        ];
    }
}
