<?php
// +----------------------------------------------------------------------
// | 消息中心，获取重发成功率图表数据（月查询）
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace app\socket\controller\api\message_center;
use think\Db;

class get_rate_chart {

    /**
     * 统一入口
     * @todo 无
     * @author 小黄牛
     * @version v1.0.9 + 2020.04.16
     * @deprecated 暂不启用
     * @global 无
     * @param swoole $ws 实例
     * @param swoole $request 请求体信息
     * @param response $response 请求内容 来源类实例也在里面
     * @return void
    */
    public function run($ws, $request, $response) {
        # 获取请求参数，不区分请求类型，POST优先
        $param = $response->class->param($request);
        # 可以这样判断请求类型
        if ($request->server['request_method'] != 'GET') {
            # 如果需要返回消息，可以这样
            $response->class->endRequest($request, $response, '301', '请求类型错误');
            # 注意，在接口中不能exit，否则会报错的
            return false;
        }
        if (empty($param['date'])) {
            $response->class->endRequest($request, $response, '301', '请输入date日期参数');
            return false;
        }

        # 获取Redis连接
        $obj = \pool\RedisPool::run()->pop();
        $redis = $obj['redis'];

        $date = $this->getLastDay($param['date'].'-01');
        $success = [];
        $error = [];

        foreach ($date as $v) {
            $day = date('Ymd', strtotime($v));
            $success_key = $ws->message_center_success.$day;
            $error_key = $ws->message_center_error.$day;

            # 读取
            $success[] = $redis->get($success_key) ?: 0;
            $error[] = $redis->get($error_key) ?: 0;
        }
       
        # 归回连接
        \pool\RedisPool::run()->free($obj);

        $response->class->endRequest($request, $response, '200', '获取成功', [
            'date' => $date,
            'success' => $success,
            'error' => $error,
        ]);
        return true;
    }

    // 获取天数
    public function getLastDay($date) {
        $time = strtotime($date);
        $last = date('j', strtotime(date('Y-m-01', $time) . ' +1 month -1 day'));
        $date = [];
        for ($i=1; $i<=$last; $i++) {
            if ($i >= 10) {
                $day = $i;
            } else {
                $day = '0'.$i;
            }
            $date[] = date('Y-m', $time).'-'.$day;
        }
        return $date;
    }

    /**
     * 该接口文档说明
     * @author 小黄牛
     * @version v1.0.1 + 2020.02.24
    */
    public function _link() {
        return [
            // 请求头
            'header' => [
                'method' => 'GET', // 请求类型
                'code' => 'utf-8', // 内容编码
                'content-type' => 'application/x-www-form-urlencoded', // 数据类型
                // 其余自定义header
            ],
            // 请求参数
            'param' => [
                // 参考格式
                'date' => '2020-05', // 结束时间 date 不可为空，只到月
            ],
            // 返回值
            'return' => [
                // 状态码囊括
                'code' => [
                    '200' => '请求正常', 
                    '301' => '错误提示', 
                    '500' => '服务器错误',
                ],
                // data部分返回值说明
                'data' => [ //
                    'date' => '', // 时间轴
                    'success' => '', // 成功数
                    'error' => '', // 失败数
                ]
            ]
        ];
    }
}
