<?php
// +----------------------------------------------------------------------
// | 监听WebSocket进程平滑重启
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace app\socket\controller\event;

class onWorkerStart {
    /**
	 * WS的启动实例
	*/
    public $_ws;
    
    // 构造函数
    public function __construct(&$ws) {
        $this->_ws = $ws;
    }

    /**
     * 统一入口
     * @todo 无
     * @author 小黄牛
     * @version v1.0.13 + 2020.05.04
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    public function run($server, $worker_id) {
        // 这里需要注意，第一次启动服务时，也会进入该流程
        
        # 启动数据库连接池
        $this->start_mysql();
        # 启动Redis连接池
        $this->start_redis();
        # 内存表创建
        $this->ram_table();
        # 创建消息中心配置参数
        $this->start_message_center();
    }

    /**
     * 创建消息中心配置参数
     * @todo 无
     * @author 小黄牛
     * @version v1.0.13 + 2020.05.04
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    private function start_message_center() {
        $this->_ws->message_center_success = $this->_ws->config['websocket_redis_message_center'].'success_';
        $this->_ws->message_center_error = $this->_ws->config['websocket_redis_message_center'].'error_';
    }
    
    /**
     * 打开Mysql连接池
     * @todo 无
     * @author 小黄牛
     * @version v1.0.13 + 2020.05.04
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    private function start_mysql() {
        // 启动数据库连接池
        \pool\MysqlPool::run()->init();
        // 启动连接池检测定时器
        \pool\MysqlPool::run()->timing_recovery($this->_ws->config['mysql_timing_recovery']);
    }
    
    /**
     * 打开Redis连接池
     * @todo 无
     * @author 小黄牛
     * @version v1.0.13 + 2020.05.04
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    private function start_redis() {
        // 启动数据库连接池
        \pool\RedisPool::run()->init();
        // 启动连接池检测定时器
        \pool\RedisPool::run()->timing_recovery($this->_ws->config['redis_timing_recovery']);
    }
    
    /**
     * 创建内存表
     * @todo 无
     * @author 小黄牛
     * @version v1.0.13 + 2020.05.04
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    private function ram_table() {
        # 创建用户在线表状态
        $user = new \swoole_table($this->_ws->config['websocket_max_online']);
        # 创建字段
        $user->column('fd', \swoole_table::TYPE_INT, 8); // FD
        $user->column('status', \swoole_table::TYPE_INT, 8); // 状态
        $user->column('heartbeat', \swoole_table::TYPE_INT, 8); // 心跳重连数
        $user->column('id', \swoole_table::TYPE_STRING, 32); // 会员ID
        $user->create();
        $this->_ws->user = $user;
    }

    /**
     * 创建Redis
     * @todo 无
     * @author 小黄牛
     * @version v1.0.13 + 2020.05.04
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    private function create_redis() {
        # 启动redis
        $redis = new \Swoole\Coroutine\Redis();
        $redis->connect($this->_ws->config['websocket_redis_host'], $this->_ws->config['websocket_redis_port']);
        # 设置Swoole-Redis配置
        $redis->setOptions([
            'connect_timeout' => $this->_ws->config['websocket_redis_connect_timeout'],
            'timeout' => $this->_ws->config['websocket_redis_timeout'],
            'serialize' => $this->_ws->config['websocket_redis_serialize'],
            'reconnect' => $this->_ws->config['websocket_redis_reconnect'],
            'compatibility_mode' => $this->_ws->config['websocket_redis_compatibility_mode'],
        ]);
        # 如果有密码
        if ($this->_ws->config['websocket_redis_pwd']) {
            $redis->auth($this->_ws->config['websocket_redis_pwd']);
        }

        if (!$redis->connected) {
            echo 'redis链接失败：'.$redis->errCode.'，msg：'.$redis->errMsg;
            exit;
        }

        $this->_ws->redis = $redis;
    }

}